<?php

/*********************************************************************

 アルバムプラグイン (2018/05/05)

 freo.jp
 customize by 32877(http://32877.xii.jp)

*********************************************************************/

//外部ファイル読み込み
require_once FREO_MAIN_DIR . 'freo/plugins/associate_plugin_album.php';
require_once FREO_MAIN_DIR . 'freo/plugins/security_plugin_album.php';
require_once FREO_MAIN_DIR . 'freo/plugins/filter_plugin_album.php';

/* メイン処理 */
function freo_page_album()
{
	global $freo;

	switch ($_REQUEST['freo']['work']) {
		case 'setup':
			freo_page_album_setup();
			break;
		case 'setup_execute':
			freo_page_album_setup_execute();
			break;
		case 'admin':
			freo_page_album_admin();
			break;
		case 'admin_form':
			freo_page_album_admin_form();
			break;
		case 'admin_post':
			freo_page_album_admin_post();
			break;
		case 'admin_file_edit':
			freo_page_album_admin_file_edit();
			break;
		case 'admin_file_post':
			freo_page_album_admin_file_post();
			break;
		case 'admin_file_delete':
			freo_page_album_admin_file_delete();
			break;
		case 'admin_delete':
			freo_page_album_admin_delete();
			break;
		case 'admin_category':
			freo_page_album_admin_category();
			break;
		case 'admin_category_form':
			freo_page_album_admin_category_form();
			break;
		case 'admin_category_post':
			freo_page_album_admin_category_post();
			break;
		case 'admin_category_update':
			freo_page_album_admin_category_update();
			break;
		case 'admin_category_delete':
			freo_page_album_admin_category_delete();
			break;
		case 'view':
			freo_page_album_view();
			break;
		default:
			freo_page_album_default();
	}

	return;
}

/* セットアップ */
function freo_page_album_setup()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//エラー確認
		if (!$freo->smarty->get_template_vars('errors')) {
			freo_redirect('album/setup_execute?freo%5Btoken%5D=' . freo_token('create'), true);
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'       => freo_token('create'),
		'plugin_id'   => 'album',
		'plugin_name' => FREO_PLUGIN_ALBUM_NAME
	));

	//データ出力
	freo_output('plugins/setup.html');

	return;
}

/* セットアップ | セットアップ実行 */
function freo_page_album_setup_execute()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('setup?error=1', true);
	}

	//データベーステーブル存在検証
	if (FREO_DATABASE_TYPE == 'mysql') {
		$query = 'SHOW TABLES';
	} else {
		$query = 'SELECT name FROM sqlite_master WHERE type = \'table\'';
	}
	$stmt = $freo->pdo->query($query);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$table = array();
	while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$table[$data[0]] = true;
	}

	//データベーステーブル定義
	if (FREO_DATABASE_TYPE == 'mysql') {
		$queries = array(
			'plugin_albums' => '(
				id INT UNSIGNED NOT NULL AUTO_INCREMENT,
				created DATETIME NOT NULL,
				modified DATETIME NOT NULL,
				title VARCHAR(255) NOT NULL,
				tag VARCHAR(255),
				text TEXT,
				category_id VARCHAR(255),
				option01    TEXT,
				option02    TEXT,
				option03    TEXT,
				option04    TEXT,
				option05    TEXT,
				option06    TEXT,
				option07    TEXT,
				option08    TEXT,
				option09    TEXT,
				option10    TEXT,
				restriction VARCHAR(20),
				password VARCHAR(80),
				PRIMARY KEY(id)
			)',
			'plugin_album_categories' => '(
				id       VARCHAR(80)  NOT NULL,
				created  DATETIME     NOT NULL,
				modified DATETIME     NOT NULL,
				sort     INT UNSIGNED NOT NULL,
				name     VARCHAR(255) NOT NULL,
				memo     TEXT,
				PRIMARY KEY(id)
			)',
			'plugin_album_filter_sets'   => '(filter_id VARCHAR(80) NOT NULL, album_id INT UNSIGNED)',
			'plugin_album_group_sets'    => '(group_id VARCHAR(80) NOT NULL, user_id VARCHAR(80), album_id INT UNSIGNED)'
		);
	} else {
		$queries = array(
			'plugin_albums' => '(
				id INTEGER,
				created DATETIME NOT NULL,
				modified DATETIME NOT NULL,
				title VARCHAR NOT NULL,
				tag VARCHAR,
				text TEXT,
				category_id VARCHAR,
				option01    TEXT,
				option02    TEXT,
				option03    TEXT,
				option04    TEXT,
				option05    TEXT,
				option06    TEXT,
				option07    TEXT,
				option08    TEXT,
				option09    TEXT,
				option10    TEXT,
				restriction VARCHAR,
				password VARCHAR,
				PRIMARY KEY(id)
			)',
			'plugin_album_categories' => '(
				id       VARCHAR          NOT NULL,
				created  DATETIME         NOT NULL,
				modified DATETIME         NOT NULL,
				sort     INTEGER UNSIGNED NOT NULL,
				name     VARCHAR          NOT NULL,
				memo     TEXT,
				PRIMARY KEY(id)
			)',
			'plugin_album_filter_sets'   => '(filter_id VARCHAR NOT NULL, album_id INTEGER UNSIGNED)',
			'plugin_album_group_sets'    => '(group_id VARCHAR NOT NULL, user_id VARCHAR, album_id INTEGER UNSIGNED)',
		);
	}

	//データベーステーブル作成
	foreach ($queries as $name => $query) {
		if (empty($table[FREO_DATABASE_PREFIX . $name])) {
			$stmt = $freo->pdo->query('CREATE TABLE ' . FREO_DATABASE_PREFIX . $name . $query);
			if (!$stmt) {
				freo_error($freo->pdo->errorInfo());
			}
		}
	}

	//ログ記録
	freo_log(FREO_PLUGIN_ALBUM_NAME . 'のセットアップを実行しました。');

	//完了画面へ移動
	freo_redirect('album/setup?exec=setup', true);

	return;
}

/* 管理画面 | アルバム管理 */
function freo_page_album_admin()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND title LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['tag'])) {
		$condition .= ' AND tag = ' . $freo->pdo->quote($_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote($_GET['tag'] . ',%') . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag'] . ',%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}
	if (isset($_GET['category_id'])) {
		$condition .= ' AND category_id = ' . $freo->pdo->quote($_GET['category_id']);
	}
	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//アルバム取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums ' . $condition . ' ORDER BY id DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['album']['admin_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['album']['admin_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_albums = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_albums[$data['id']] = $data;
	}

	//アルバムID取得
	$plugin_album_keys = array_keys($plugin_albums);

	//ファイル取得
	$plugin_album_files = array();
	foreach ($plugin_album_keys as $plugin_album) {
		$file_dir = FREO_FILE_DIR . 'plugins/album_files/' . $plugin_album . '/';
		$thumbnail_dir = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $plugin_album . '/';

		if (file_exists($file_dir)) {
			if ($dir = scandir($file_dir)) {
				foreach ($dir as $data) {
					if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
						list($width, $height, $size) = freo_file($file_dir . $data);
						if (is_file($thumbnail_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
							list($thumbnailwidth, $thumbnailheight, $thumbnailsize) = freo_file($thumbnail_dir . $data);
							$thumbnail = array(
								'name'    => $data,
								'width'   => $thumbnailwidth,
								'height'  => $thumbnailheight,
								'size'    => $thumbnailsize
							);
						} else {
							$thumbnail = array();
						}
						$plugin_album_files[$plugin_album][$matches[1]] = array(
							'name'      => $data,
							'width'     => $width,
							'height'    => $height,
							'size'      => $size,
							'thumbnail' => $thumbnail
						);
					}
				}
			} else {
				freo_error('ファイル保存ディレクトリを開けません。');
			}
		}
	}

	//アルバム数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                  = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_album_count = $data[0];
	$plugin_album_page  = ceil($plugin_album_count / $freo->config['plugin']['album']['admin_limit']);

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_album_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                 => freo_token('create'),
		'plugin_albums'      => $plugin_albums,
		'plugin_album_categories' => $plugin_album_categories,
		'plugin_album_count' => $plugin_album_count,
		'plugin_album_files' => $plugin_album_files,
		'plugin_album_page'  => $plugin_album_page,
	));

	if (isset($_GET['type']) and $_GET['type'] == 'iframe') {
		//データ出力
		freo_output('plugins/album/iframe_album.html');
	}

	return;
}

/* 管理画面 | アルバム入力 */
function freo_page_album_admin_form()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		$_GET['id'] = 0;
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//アップロード項目数取得
		$file_count = count($_FILES['plugin_album']['tmp_name']['file']);

		//アップロードデータ初期化
		for ($i = 0; $i < $file_count; $i++) {
			if (!isset($_FILES['plugin_album']['tmp_name']['file'][$i])) {
				$_FILES['plugin_album']['tmp_name']['file'][$i] = null;
			}
		}

		//アップロードデータ取得
		for ($i = 0; $i < $file_count; $i++) {
			if (is_uploaded_file($_FILES['plugin_album']['tmp_name']['file'][$i])) {
				$_POST['plugin_album']['file'][$i] = $_FILES['plugin_album']['name']['file'][$i];
			} else {
				$_POST['plugin_album']['file'][$i] = null;
			}
		}

		//関連データ取得
		if (isset($_POST['plugin_album_associate'])) {
		} else {
			$_POST['plugin_album_associate'] = array();
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {

			//タイトル
			if ($_POST['plugin_album']['title'] == '') {
				$freo->smarty->append('errors', 'タイトルが入力されていません。');
			} elseif (mb_strlen($_POST['plugin_album']['title'], 'UTF-8') > 200) {
				$freo->smarty->append('errors', 'タイトルは200文字以内で入力してください。');
			}

			//タグ
			if (mb_strlen($_POST['plugin_album']['tag'], 'UTF-8') > 80) {
				$freo->smarty->append('errors', 'タグは80文字以内で入力してください。');
			}

			//本文
			if (mb_strlen($_POST['plugin_album']['text'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', '本文は5000文字以内で入力してください。');
			}

			//ファイル
			if ($file_count > 0) {
				$filenames = array();
				//ファイル名
				for ($i = 0; $i < $file_count; $i++) {
					if ($_POST['plugin_album']['file'][$i] == '') {
						continue;
					}
					if ($_POST['plugin_album']['file'][$i] == '') {
						$freo->smarty->append('errors', $_POST['plugin_album']['file'][$i] . ' ファイルのファイル名が入力されていません。');
					} elseif (isset($filenames[$_POST['plugin_album']['file'][$i]])) {
						$freo->smarty->append('errors', $_POST['plugin_album']['file'][$i] . ' ファイル名はすべて異なるものを入力してください。');
					} elseif (!$freo->config['plugin']['album']['filename'] and !preg_match('/^[\w\.\~\-\&\#\+\=\;\@\%]+$/', $_POST['plugin_album']['file'][$i])) {
						$freo->smarty->append('errors', $_POST['plugin_album']['file'][$i] . ' ファイル名は半角英数字で入力してください。');
					} elseif (!$freo->config['plugin']['album']['filename'] and mb_strlen($_POST['plugin_album']['file'][$i], 'UTF-8') > 80) {
						$freo->smarty->append('errors', $_POST['plugin_album']['file'][$i] . ' ファイル名は80文字以内で入力してください。');
					} elseif (!$freo->config['plugin']['album']['filename'] and file_exists(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/' . $_POST['plugin_album']['file'][$i])) {
						$freo->smarty->append('errors', $_POST['plugin_album']['file'][$i] . ' はすでにアップロードされています。');
					} elseif (!preg_match('/\.(gif|jpeg|jpg|jpe|png)$/i', $_POST['plugin_album']['file'][$i])) {
						$freo->smarty->append('errors', $_POST['plugin_album']['file'][$i] . ' アップロードできるイメージはGIF、JPEG、PNGのみです。');
					}
					$filenames[$_POST['plugin_album']['file'][$i]] = true;
				}
			}
		}
		

		//ファイルアップロード
		for ($i = 0; $i < $file_count; $i++) {
			$file_flag = false;

			if (!$freo->smarty->get_template_vars('errors')) {
				if (is_uploaded_file($_FILES['plugin_album']['tmp_name']['file'][$i])) {
					$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/album_files/';

					if (move_uploaded_file($_FILES['plugin_album']['tmp_name']['file'][$i], $temporary_dir . $_FILES['plugin_album']['name']['file'][$i])) {
						chmod($temporary_dir . $_FILES['plugin_album']['name']['file'][$i], FREO_PERMISSION_FILE);

						if (preg_match('/' . FREO_ASCII_FILE . '/i', $_FILES['plugin_album']['name']['file'][$i])) {
							$data = file_get_contents($temporary_dir . $_FILES['plugin_album']['name']['file'][$i]);

							if ($data) {
								if (file_put_contents($temporary_dir . $_FILES['plugin_album']['name']['file'][$i], freo_unify($data)) === false) {
									$freo->smarty->append('errors', 'ファイルの改行コードを変更できません。');
								}
							}
						}

						$file_flag = true;
					} else {
						$freo->smarty->append('errors', 'ファイルをアップロードできません。');
					}

					if ($file_flag) {
						if ($freo->config['plugin']['album']['thumbnail']) {
							$thumbnail_width  = $freo->config['plugin']['album']['thumbnail_width'];
							$thumbnail_height = $freo->config['plugin']['album']['thumbnail_height'];

							freo_resize($temporary_dir . $_FILES['plugin_album']['name']['file'][$i], FREO_FILE_DIR . 'temporaries/plugins/album_thumbnails/' . $_FILES['plugin_album']['name']['file'][$i], $thumbnail_width, $thumbnail_height);
						}
						if ($freo->config['plugin']['album']['original']) {
							freo_resize($temporary_dir . $_FILES['plugin_album']['name']['file'][$i], $temporary_dir . $_FILES['plugin_album']['name']['file'][$i], $freo->config['plugin']['album']['original_width'], $freo->config['plugin']['album']['original_height']);
						}
					}
				}
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_album = $_POST['plugin_album'];
			$plugin_album_associate = $_POST['plugin_album_associate'];
		} else {
			$_SESSION['input'] = $_POST;

			//登録処理へ移動
			freo_redirect('album/admin_post?freo%5Btoken%5D=' . freo_token('create') . ($_GET['id'] ? '&id=' . $_GET['id'] : ''));
		}
	} else {
		if ($_GET['id']) {
			//編集データ取得
			$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums WHERE id = :id');
			$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}

			if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$plugin_album = $data;
			} else {
				freo_error('指定されたアルバムが見つかりません。');
			}

			//関連データ取得
			$plugin_album_associates = freo_associate_plugin_album('get', array($_GET['id']));
			$plugin_album_associate  = $plugin_album_associates[$_GET['id']];

			//表示順
			$sort = 0;
			if ($freo->config['plugin']['album']['album_file_sort'] == 'descend') {
				$sort = 1;
			}

			//ファイル取得
			$plugin_album_files = array();

			if (file_exists(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'])) {
				if (($dir = scandir(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'], $sort))) {

					foreach ($dir as $data) {

						if ($data == '.' or $data == '..') {
							continue;
						}

						list($width, $height, $size) = freo_file(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/' . $data);

						if (is_file(FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/' . $data)) {
							list($thumbnail_width, $thumbnail_height, $thumbnail_size) = freo_file(FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/' . $data);

							$thumbnail = array(
								'name'     => $data,
								'width'    => $thumbnail_width,
								'height'   => $thumbnail_height,
								'size'     => $thumbnail_size,
							);
						} else {
							$thumbnail = array();
						}

						$plugin_album_files[] = array(
							'name'      => $data,
							'width'     => $width,
							'height'    => $height,
							'size'      => $size,
							'thumbnail' => $thumbnail,
						);
					}
				} else {
					freo_error('アルバムファイル格納ディレクトリ ' . FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . ' を開けません。');
				}

				//データ割当
				$freo->smarty->assign(array(
					'plugin_album_files' => $plugin_album_files,
				));
			}

		} else {
			//新規データ設定
			$plugin_album = array();
			$plugin_album_associate = array();
		}
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_album_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_categories[$data['id']] = $data;
	}


	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'plugin_album_categories' => $plugin_album_categories,
		'input' => array(
			'plugin_album' => $plugin_album,
			'plugin_album_associate' => $plugin_album_associate
		)
	));

	return;
}

/* 管理画面 | アルバム登録 */
function freo_page_album_admin_post()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('album/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('album/admin?error=1');
	}

	//入力データ取得
	$album                  = $_SESSION['input']['plugin_album'];
	$plugin_album_associate = $_SESSION['input']['plugin_album_associate'];

	if ($album['category_id'] == '') {
		$album['category_id'] = null;
	}

	if ($album['tag'] == '') {
		$album['tag'] = null;
	}
	if ($album['text'] == '') {
		$album['text'] = null;
	}
	if ($album['option01'] == '') {
		$album['option01'] = null;
	}
	if ($album['option02'] == '') {
		$album['option02'] = null;
	}
	if ($album['option03'] == '') {
		$album['option03'] = null;
	}
	if ($album['option04'] == '') {
		$album['option04'] = null;
	}
	if ($album['option05'] == '') {
		$album['option05'] = null;
	}
	if ($album['option06'] == '') {
		$album['option06'] = null;
	}
	if ($album['option07'] == '') {
		$album['option07'] = null;
	}
	if ($album['option08'] == '') {
		$album['option08'] = null;
	}
	if ($album['option09'] == '') {
		$album['option09'] = null;
	}
	if ($album['option10'] == '') {
		$album['option10'] = null;
	}
	if ($album['restriction'] == '') {
		$album['restriction'] = null;
	}
	if ($album['password'] == '') {
		$album['password'] = null;
	}

	//データ登録
	if (isset($_GET['id'])) {
		$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_albums SET modified = :now, title = :title, tag = :tag, text = :text, category_id = :category_id, option01 = :option01, option02 = :option02, option03 = :option03, option04 = :option04, option05 = :option05, option06 = :option06, option07 = :option07, option08 = :option08, option09 = :option09, option10 = :option10, restriction = :restriction, password = :password WHERE id = :id');
		$stmt->bindValue(':now',             date('Y-m-d H:i:s'));
		$stmt->bindValue(':title',           $album['title']);
		$stmt->bindValue(':tag',             $album['tag']);
		$stmt->bindValue(':text',            $album['text']);
		$stmt->bindValue(':category_id',     $album['category_id']);
		$stmt->bindValue(':option01',        $album['option01']);
		$stmt->bindValue(':option02',        $album['option02']);
		$stmt->bindValue(':option03',        $album['option03']);
		$stmt->bindValue(':option04',        $album['option04']);
		$stmt->bindValue(':option05',        $album['option05']);
		$stmt->bindValue(':option06',        $album['option06']);
		$stmt->bindValue(':option07',        $album['option07']);
		$stmt->bindValue(':option08',        $album['option08']);
		$stmt->bindValue(':option09',        $album['option09']);
		$stmt->bindValue(':option10',        $album['option10']);
		$stmt->bindValue(':restriction',     $album['restriction']);
		$stmt->bindValue(':password',        $album['password']);
		$stmt->bindValue(':id',              $album['id'], PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
		$plugin_album_associate['id'] = $album['id'];
	} else {
		$stmt = $freo->pdo->prepare('INSERT INTO ' . FREO_DATABASE_PREFIX . 'plugin_albums VALUES(NULL, :now1, :now2, :title, :tag, :text, :category_id, :option01, :option02, :option03, :option04, :option05, :option06, :option07, :option08, :option09, :option10, :restriction, :password)');
		$stmt->bindValue(':now1',        date('Y-m-d H:i:s'));
		$stmt->bindValue(':now2',        date('Y-m-d H:i:s'));
		$stmt->bindValue(':title',       $album['title']);
		$stmt->bindValue(':tag',         $album['tag']);
		$stmt->bindValue(':text',        $album['text']);
		$stmt->bindValue(':category_id', $album['category_id']);
		$stmt->bindValue(':option01',    $album['option01']);
		$stmt->bindValue(':option02',    $album['option02']);
		$stmt->bindValue(':option03',    $album['option03']);
		$stmt->bindValue(':option04',    $album['option04']);
		$stmt->bindValue(':option05',    $album['option05']);
		$stmt->bindValue(':option06',    $album['option06']);
		$stmt->bindValue(':option07',    $album['option07']);
		$stmt->bindValue(':option08',    $album['option08']);
		$stmt->bindValue(':option09',    $album['option09']);
		$stmt->bindValue(':option10',    $album['option10']);
		$stmt->bindValue(':restriction', $album['restriction']);
		$stmt->bindValue(':password',    $album['password']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
		$album['id']           = $freo->pdo->lastInsertId();
		$plugin_album_associate['id'] = $freo->pdo->lastInsertId();
	}

	//ファイル保存
	$file_dir      = FREO_FILE_DIR . 'plugins/album_files/' . $album['id'] . '/';
	$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/album_files/';
	$thumbnail_dir = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $album['id'] . '/';

	//アップロード項目数取得
	$file_count = count($album['file']);

	//現在日時取得
	$now = time();
	
	//ファイル保存
	for ($i = 0; $i < $file_count; $i++) {
		if ($album['file'][$i] != '') {
			$org_album = $album['file'][$i];
			if (!freo_mkdir($file_dir, FREO_PERMISSION_DIR)) {
				freo_error('ディレクトリ ' . $file_dir . ' を作成できません。');
			}
			if (rename(FREO_FILE_DIR . 'temporaries/plugins/album_files/' . $album['file'][$i], $file_dir . $album['file'][$i])) {
				if ($freo->config['plugin']['album']['filename'] and preg_match('/\.(.*)$/', $album['file'][$i], $matches)) {
					$filename = 'file' . date('YmdHis', $now + $i) . '.' . $matches[1];

					if (rename($file_dir . $album['file'][$i], $file_dir . $filename)) {
						$album['file'][$i] = $filename;
					} else {
						freo_error('ファイル ' . $file_dir . $album['file'][$i] . ' の名前を変更できません。');
					}
				}

				chmod($file_dir . $album['file'][$i], FREO_PERMISSION_FILE);
			} else {
				freo_error('ファイル ' . FREO_FILE_DIR . 'temporaries/plugins/album_file/' . $album['file'][$i] . ' を移動できません。');
			}

			if ($freo->config['plugin']['album']['thumbnail']) {
				$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/album_thumbnails/';

				if ($org_album and file_exists($temporary_dir . $org_album)) {
					if (!freo_mkdir($thumbnail_dir, FREO_PERMISSION_DIR)) {
						freo_error('ディレクトリ ' . $thumbnail_dir . ' を作成できません。');
					}

					if (rename($temporary_dir . $org_album, $thumbnail_dir . $album['file'][$i])) {
						chmod($thumbnail_dir . $album['file'][$i], FREO_PERMISSION_FILE);
					} else {
						freo_error('ファイル ' . $temporary_dir . $org_album . ' を移動できません。');
					}
				}
			}
		}
	}

	//関連データ更新
	freo_associate_plugin_album('post', $plugin_album_associate);

	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	if (isset($_GET['id'])) {
		freo_log('アルバムを編集しました。');
	} else {
		freo_log('アルバムを新規に登録しました。');
	}

	//アルバム管理へ移動
	if (isset($_GET['id'])) {
		freo_redirect('album/admin?exec=update&id=' . $album['id']);
	} else {
		freo_redirect('album/admin?exec=insert');
	}

	return;
}

/* 管理画面 | ファイル編集 */
function freo_page_album_admin_file_edit()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('album/admin?error=1');
	}

	//パラメータ検証
	if (!isset($_GET['name'])) {
		freo_redirect('album/admin_form?id=' . $_GET['id'] . '&error=1');
	}
	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {

		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//アップロードデータ初期化
		if (!isset($_FILES['plugin_album']['tmp_name']['file'])) {
			$_FILES['plugin_album']['tmp_name']['file'] = null;
		}

		//アップロードデータ取得
		if (is_uploaded_file($_FILES['plugin_album']['tmp_name']['file'])) {
			$_POST['plugin_album']['file'] = $_FILES['plugin_album']['name']['file'];
		} else {
			$_POST['plugin_album']['file']= null;
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {
			//ファイル名
			if (isset($_POST['plugin_album']['newname'])) {
				if (!$freo->config['plugin']['album']['filename'] and !preg_match('/^[\w\.\~\-\&\#\+\=\;\@\%]+$/', $_POST['plugin_album']['newname'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['newname'] . ' ファイル名は半角英数字で入力してください。');
				} elseif (!$freo->config['plugin']['album']['filename'] and mb_strlen($_POST['plugin_album']['newname'], 'UTF-8') > 80) {
					$freo->smarty->append('errors', $_POST['plugin_album']['newname'] . ' ファイル名は80文字以内で入力してください。');
				} elseif (!$freo->config['plugin']['album']['filename'] and file_exists(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/' . $_POST['plugin_album']['newname'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['newname'] . ' はすでにアップロードされています。');
				} elseif (!preg_match('/\.(gif|jpeg|jpg|jpe|png)$/i', $_POST['plugin_album']['newname'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['newname'] . ' アップロードできるイメージはGIF、JPEG、PNGのみです。');
				}
			}

			//ファイル
			if (isset($_POST['plugin_album']['file'])) {
				if ($_POST['plugin_album']['file'] == '') {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' ファイルのファイル名が入力されていません。');
				} elseif (!preg_match('/\.(gif|jpeg|jpg|jpe|png)$/i', $_POST['plugin_album']['file'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' アップロードできるイメージはGIF、JPEG、PNGのみです。');
				} elseif (preg_match('/\.gif$/i', $_POST['plugin_album']['file_org']) and !preg_match('/\.gif$/i', $_POST['plugin_album']['file'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' 差し替えファイルの拡張子が違います。');
				} elseif (preg_match('/\.jpeg$/i', $_POST['plugin_album']['file_org']) and !preg_match('/\.jpeg$/i', $_POST['plugin_album']['file'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' 差し替えファイルの拡張子が違います。');
				} elseif (preg_match('/\.jpg$/i', $_POST['plugin_album']['file_org']) and !preg_match('/\.jpg$/i', $_POST['plugin_album']['file'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' 差し替えファイルの拡張子が違います。');
				} elseif (preg_match('/\.jpe$/i', $_POST['plugin_album']['file_org']) and !preg_match('/\.jpe$/i', $_POST['plugin_album']['file'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' 差し替えファイルの拡張子が違います。');
				} elseif (preg_match('/\.png$/i', $_POST['plugin_album']['file_org']) and !preg_match('/\.png$/i', $_POST['plugin_album']['file'])) {
					$freo->smarty->append('errors', $_POST['plugin_album']['file'] . ' 差し替えファイルの拡張子が違います。');
				}
			}
		}
		
		//ファイルアップロード
		$file_flag = false;

		if (!$freo->smarty->get_template_vars('errors')) {
			if (is_uploaded_file($_FILES['plugin_album']['tmp_name']['file'])) {
				$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/album_files/';

				if (move_uploaded_file($_FILES['plugin_album']['tmp_name']['file'], $temporary_dir . $_FILES['plugin_album']['name']['file'])) {
					chmod($temporary_dir . $_FILES['plugin_album']['name']['file'], FREO_PERMISSION_FILE);

					if (preg_match('/' . FREO_ASCII_FILE . '/i', $_FILES['plugin_album']['name']['file'])) {
						$data = file_get_contents($temporary_dir . $_FILES['plugin_album']['name']['file']);

						if ($data) {
							if (file_put_contents($temporary_dir . $_FILES['plugin_album']['name']['file'], freo_unify($data)) === false) {
								$freo->smarty->append('errors', 'ファイルの改行コードを変更できません。');
							}
						}
					}
					$file_flag = true;
				} else {
					$freo->smarty->append('errors', 'ファイルをアップロードできません。');
				}

				if ($file_flag) {
					$deletethumbnail = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/' . $_POST['plugin_album']['file_org'];
					if (file_exists($deletethumbnail)) {
						unlink($deletethumbnail);
					}
					if ($freo->config['plugin']['album']['thumbnail']) {
						$thumbnail_width  = $freo->config['plugin']['album']['thumbnail_width'];
						$thumbnail_height = $freo->config['plugin']['album']['thumbnail_height'];

						freo_resize($temporary_dir . $_FILES['plugin_album']['name']['file'], FREO_FILE_DIR . 'temporaries/plugins/album_thumbnails/' . $_FILES['plugin_album']['name']['file'], $thumbnail_width, $thumbnail_height);
					}
					if ($freo->config['plugin']['album']['original']) {
						freo_resize($temporary_dir . $_FILES['plugin_album']['name']['file'], $temporary_dir . $_FILES['plugin_album']['name']['file'], $freo->config['plugin']['album']['original_width'], $freo->config['plugin']['album']['original_height']);
					}
				}
			}
		}

		//ファイル名変更
		if (isset($_POST['plugin_album']['newname'])) {
			$file_dir      = FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/';
			$thumbnail_dir = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/';
			if (!rename($file_dir . $_POST['plugin_album']['file_org'], $file_dir . $_POST['plugin_album']['newname'])) {
				freo_error('ファイル ' . $file_dir . $_POST['plugin_album']['file_org'] . ' の名前を変更できません。');
			} elseif(file_exists($thumbnail_dir . $_POST['plugin_album']['file_org'])) {
				rename($thumbnail_dir . $_POST['plugin_album']['file_org'], $thumbnail_dir . $_POST['plugin_album']['newname']);
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_album = $_POST['plugin_album'];
		} else {
			$_SESSION['input'] = $_POST;

			//登録処理へ移動
			freo_redirect('album/admin_file_post?freo%5Btoken%5D=' . freo_token('create') . ($_GET['id'] ? '&id=' . $_GET['id'] : ''));
		}
	}

	//ファイル取得
	$editfile = FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/' . $_GET['name'];
	$plugin_album_file = array();

	if (file_exists($editfile)) {

		list($width, $height, $size) = freo_file($editfile);
		$plugin_album_file = array(
			'name'      => $_GET['name'],
			'width'     => $width,
			'height'    => $height,
			'size'      => $size,
		);
	} else {
		freo_error('アルバムファイル格納ディレクトリ ' . FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . ' を開けません。');
	}

	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'plugin_album_file' => $plugin_album_file,
	));
	return;
}
/* 管理画面 | アルバムファイル差し替え */
function freo_page_album_admin_file_post()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('album/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('album/admin?error=1');
	}

	//入力データ取得
	$album                  = $_SESSION['input']['plugin_album'];

	//ファイル保存
	$file_dir      = FREO_FILE_DIR . 'plugins/album_files/' . $album['id'] . '/';
	$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/album_files/';
	$thumbnail_dir = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $album['id'] . '/';


	if ($album['file'] != '') {
		$org_album = $album['file'];
		if (!freo_mkdir($file_dir, FREO_PERMISSION_DIR)) {
			freo_error('ディレクトリ ' . $file_dir . ' を作成できません。');
		}
		if (rename(FREO_FILE_DIR . 'temporaries/plugins/album_files/' . $album['file'], $file_dir . $album['file_org'])) {
			chmod($file_dir . $album['file_org'], FREO_PERMISSION_FILE);
		} else {
			freo_error('ファイル ' . FREO_FILE_DIR . 'temporaries/plugins/album_file/' . $album['file_org'] . ' を移動できません。');
		}

		if ($freo->config['plugin']['album']['thumbnail']) {
			$temporary_dir = FREO_FILE_DIR . 'temporaries/plugins/album_thumbnails/';

			if ($org_album and file_exists($temporary_dir . $org_album)) {
				if (!freo_mkdir($thumbnail_dir, FREO_PERMISSION_DIR)) {
					freo_error('ディレクトリ ' . $thumbnail_dir . ' を作成できません。');
				}

				if (rename($temporary_dir . $org_album, $thumbnail_dir . $album['file_org'])) {
					chmod($thumbnail_dir . $album['file_org'], FREO_PERMISSION_FILE);
				} else {
					freo_error('ファイル ' . $temporary_dir . $album['file_org'] . ' を移動できません。');
				}
			}
		}
	}


	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	if (isset($_GET['id'])) {
		freo_log('アルバムファイルを編集しました。');
	} else {
		freo_log('アルバムを新規に登録しました。');
	}

	//アルバム編集画面へ移動
	if (isset($_GET['id'])) {
		freo_redirect('album/admin_form?file_edit=complate&id=' . $album['id']);
	} else {
		freo_redirect('album/admin');
	}

	return;
}

/* 管理画面 | ファイル削除 */
function freo_page_album_admin_file_delete()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('album/admin?error=1');
	}

	//パラメータ検証
	if (!isset($_GET['name'])) {
		freo_redirect('album/admin_form?id=' . $_GET['id'] . '&error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check') and !isset($_GET['id'])) {
		freo_redirect('album/admin_form?id=' . $_GET['id'] . '&error=1');
	} elseif (!freo_token('check')) {
		freo_redirect('album/admin?error=1');
	}

	//ファイル削除
	$deletefile = FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/' . $_GET['name'];
	$deletethumbnail = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/' . $_GET['name'];
	if (file_exists($deletefile)) {
		unlink($deletefile);
	}
	if (file_exists($deletethumbnail)) {
		unlink($deletethumbnail);
	}

	//ログ記録
	freo_log('アルバムファイル ' . $_GET['name'] . 'を削除しました。');

	//アルバム管理へ移動
	freo_redirect('album/admin_form?id=' . $_GET['id'] . '&exec=file_delete');

	return;
}

/* 管理画面 | アルバム削除 */
function freo_page_album_admin_delete()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('album/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('album/admin?error=1');
	}

	//ファイル削除
	freo_rmdir(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/');
	freo_rmdir(FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/');

	//アルバム削除
	$stmt = $freo->pdo->prepare('DELETE FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//連番リセット
	if (FREO_DATABASE_TYPE == 'mysql') {
		$stmt = $freo->pdo->query('ALTER TABLE ' . FREO_DATABASE_PREFIX . 'plugin_albums AUTO_INCREMENT = 0');
		if (!$stmt) {
			freo_error($freo->pdo->errorInfo());
		}
	}

	//ログ記録
	freo_log('アルバムを削除しました。');

	//アルバム管理へ移動
	freo_redirect('album/admin?exec=delete&id=' . $_GET['id']);

	return;
}

/* 管理画面 | カテゴリー管理 */
function freo_page_album_admin_category()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_album_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                    => freo_token('create'),
		'plugin_album_categories' => $plugin_album_categories
	));

	return;
}

/* 管理画面 | カテゴリー入力 */
function freo_page_album_admin_category_form()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^[\w\-]+$/', $_GET['id'])) {
		$_GET['id'] = null;
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//並び順取得
		if ($_POST['plugin_album_category']['sort'] != '') {
			$_POST['plugin_album_category']['sort'] = mb_convert_kana($_POST['plugin_album_category']['sort'], 'n', 'UTF-8');
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {
			//カテゴリーID
			if ($_POST['plugin_album_category']['id'] == '') {
				$freo->smarty->append('errors', 'カテゴリーIDが入力されていません。');
			} elseif (!preg_match('/^[\w\-]+$/', $_POST['plugin_album_category']['id'])) {
				$freo->smarty->append('errors', 'カテゴリーIDは半角英数字で入力してください。');
			} elseif (mb_strlen($_POST['plugin_album_category']['id'], 'UTF-8') > 80) {
				$freo->smarty->append('errors', 'カテゴリーIDは80文字以内で入力してください。');
			}

			//並び順
			if ($_POST['plugin_album_category']['sort'] == '') {
				$freo->smarty->append('errors', '並び順が入力されていません。');
			} elseif (!preg_match('/^[\d]+$/', $_POST['plugin_album_category']['sort'])) {
				$freo->smarty->append('errors', '並び順は半角英数字で入力してください。');
			} elseif (mb_strlen($_POST['plugin_album_category']['sort'], 'UTF-8') > 10) {
				$freo->smarty->append('errors', '並び順は10文字以内で入力してください。');
			}

			//カテゴリー名
			if ($_POST['plugin_album_category']['name'] == '') {
				$freo->smarty->append('errors', 'カテゴリー名が入力されていません。');
			} elseif (mb_strlen($_POST['plugin_album_category']['name'], 'UTF-8') > 80) {
				$freo->smarty->append('errors', 'カテゴリー名は80文字以内で入力してください。');
			}

			//説明
			if (mb_strlen($_POST['plugin_album_category']['memo'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', '説明は5000文字以内で入力してください。');
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_album_category = $_POST['plugin_album_category'];
		} else {
			$_SESSION['input'] = $_POST;

			//登録処理へ移動
			freo_redirect('album/admin_category_post?freo%5Btoken%5D=' . freo_token('create') . ($_GET['id'] ? '&id=' . $_GET['id'] : ''));
		}
	} else {
		if (!empty($_GET['session']) and !empty($_SESSION['input'])) {
			//入力データ復元
			$plugin_album_category = $_SESSION['input']['plugin_album_category'];
		} elseif ($_GET['id']) {
			//編集データ取得
			$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories WHERE id = :id');
			$stmt->bindValue(':id', $_GET['id']);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}

			if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$plugin_album_category = $data;
			} else {
				freo_error('指定されたカテゴリーが見つかりません。', '404 Not Found');
			}
		} else {
			//並び順初期値取得
			$stmt = $freo->pdo->query('SELECT MAX(sort) FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories');
			if (!$stmt) {
				freo_error($freo->pdo->errorInfo());
			}
			$data = $stmt->fetch(PDO::FETCH_NUM);
			$sort = $data[0] + 1;

			//新規データ設定
			$plugin_album_category = array(
				'sort' => $sort
			);
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'input' => array(
			'plugin_album_category' => $plugin_album_category
		)
	));

	return;
}

/* 管理画面 | カテゴリー登録 */
function freo_page_album_admin_category_post()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('album/admin_category?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('album/admin_category?error=1');
	}

	//入力データ取得
	$plugin_album_category = $_SESSION['input']['plugin_album_category'];

	if ($plugin_album_category['memo'] == '') {
		$plugin_album_category['memo'] = null;
	}

	//データ登録
	if (isset($_GET['id'])) {
		$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_album_categories SET modified = :now, sort = :sort, name = :name, memo = :memo WHERE id = :id');
		$stmt->bindValue(':now',  date('Y-m-d H:i:s'));
		$stmt->bindValue(':sort', $plugin_album_category['sort'], PDO::PARAM_INT);
		$stmt->bindValue(':name', $plugin_album_category['name']);
		$stmt->bindValue(':memo', $plugin_album_category['memo']);
		$stmt->bindValue(':id',   $plugin_album_category['id']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
	} else {
		$stmt = $freo->pdo->prepare('INSERT INTO ' . FREO_DATABASE_PREFIX . 'plugin_album_categories VALUES(:id, :now1, :now2, :sort, :name, :memo)');
		$stmt->bindValue(':id',   $plugin_album_category['id']);
		$stmt->bindValue(':now1', date('Y-m-d H:i:s'));
		$stmt->bindValue(':now2', date('Y-m-d H:i:s'));
		$stmt->bindValue(':sort', $plugin_album_category['sort'], PDO::PARAM_INT);
		$stmt->bindValue(':name', $plugin_album_category['name']);
		$stmt->bindValue(':memo', $plugin_album_category['memo']);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}
	}

	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	if (isset($_GET['id'])) {
		freo_log('カテゴリーを編集しました。');
	} else {
		freo_log('カテゴリーを新規に登録しました。');
	}

	//カテゴリー管理へ移動
	if (isset($_GET['id'])) {
		freo_redirect('album/admin_category?exec=update&id=' . $plugin_album_category['id']);
	} else {
		freo_redirect('album/admin_category?exec=insert');
	}

	return;
}

/* 管理画面 | カテゴリー一括編集 */
function freo_page_album_admin_category_update()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('album/admin_category?error=1');
	}

	//データ登録
	if (isset($_POST['sort'])) {
		foreach ($_POST['sort'] as $id => $sort) {
			if (!preg_match('/^[\w\-\/]+$/', $id)) {
				continue;
			}
			if (!preg_match('/^\d+$/', $sort)) {
				continue;
			}

			$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_album_categories SET sort = :sort WHERE id = :id');
			$stmt->bindValue(':sort', $sort, PDO::PARAM_INT);
			$stmt->bindValue(':id',   $id);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}
		}
	}

	//ログ記録
	freo_log('カテゴリーを並び替えました。');

	//カテゴリー管理へ移動
	freo_redirect('album/admin_category?exec=sort');

	return;
}

/* 管理画面 | カテゴリー削除 */
function freo_page_album_admin_category_delete()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^[\w\-]+$/', $_GET['id'])) {
		freo_redirect('album/admin_category?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('album/admin_category?error=1');
	}

	//カテゴリー削除
	$stmt = $freo->pdo->prepare('DELETE FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id']);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//ログ記録
	freo_log('カテゴリーを削除しました。');

	//カテゴリー管理へ移動
	freo_redirect('album/admin_category?exec=delete&id=' . $_GET['id']);

	return;
}

/* アルバム個別表示 */
function freo_page_album_view()
{
	global $freo;

	//パラメータ検証
	if (!isset($_GET['id']) and isset($freo->parameters[2])) {
		$_GET['id'] = $freo->parameters[2];
	}
	if (!isset($_GET['id']) or !preg_match('/^[\w\-]+$/', $_GET['id'])) {
		freo_error('表示したいアルバムを指定してください。');
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		if (isset($_POST['plugin_album'])) {
			//パスワード認証
			$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums WHERE id = :id AND restriction = \'password\' AND password = :password');
			$stmt->bindValue(':id',       $_GET['id'], PDO::PARAM_INT);
			$stmt->bindValue(':password', $_POST['plugin_album']['password']);
			$flag = $stmt->execute();
			if (!$flag) {
				freo_error($stmt->errorInfo());
			}

			if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
				$_SESSION['security']['plugin_album'][$_GET['id']] = true;

				freo_redirect('album/view/' . $_GET['id']);
			} else {
				freo_error('パスワードが違います。');
			}
		} else {
			//ワンタイムトークン確認
			if (!freo_token('check')) {
				$freo->smarty->append('errors', '不正なアクセスです。');
			}
		}
	}

	//アルバム取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id']);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album = $data;
	} else {
		freo_error('指定されたアルバムが見つかりません。');
	}

	//アルバムタグ取得
	if ($plugin_album['tag']) {
		$plugin_album_tags = explode(',', $plugin_album['tag']);
	} else {
		$plugin_album_tags = array();
	}

	//表示順
	$sort = 0;
	if ($freo->config['plugin']['album']['album_file_sort'] == 'descend') {
		$sort = 1;
	}

	//アルバムファイル取得
	$plugin_album_files = array();
	if ($dir = scandir(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'], $sort)) {
		natcasesort($dir);

		foreach ($dir as $data) {

			if ($data == '.' or $data == '..') {
				continue;
			}

			list($width, $height, $size) = freo_file(FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . '/' . $data);

			if (is_file(FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/' . $data)) {
				list($thumbnail_width, $thumbnail_height, $thumbnail_size) = freo_file(FREO_FILE_DIR . 'plugins/album_thumbnails/' . $_GET['id'] . '/' . $data);

				$thumbnail = array(
					'name'     => $data,
					'width'    => $thumbnail_width,
					'height'   => $thumbnail_height,
					'size'     => $thumbnail_size,
				);
			} else {
				$thumbnail = array();
			}

			$plugin_album_files[] = array(
				'name'      => $data,
				'width'     => $width,
				'height'    => $height,
				'size'      => $size,
				'thumbnail' => $thumbnail,
			);
		}
	} else {
		freo_error('アルバムファイル格納ディレクトリ ' . FREO_FILE_DIR . 'plugins/album_files/' . $_GET['id'] . ' を開けません。');
	}

	//アルバム関連データ取得
	$plugin_album_associates = freo_associate_plugin_album('get', array($_GET['id']));
	$plugin_album_associate  = $plugin_album_associates[$_GET['id']];

	//アルバムフィルター取得
	$plugin_album_filters = freo_filter_plugin_album('user', array($_GET['id']));
	$plugin_album_filter  = $plugin_album_filters[$_GET['id']];

	if ($plugin_album_filter) {
		$plugin_album['title']     = str_replace('[$title]', $plugin_album['title'], $freo->config['plugin']['album']['filter_title']);
		$plugin_album['file']      = null;
		$plugin_album_files        = null;
		$plugin_album['text']      = str_replace('[$text]', $plugin_album['text'], $freo->config['plugin']['album']['filter_text']);
	}

	//アルバム保護データ取得
	$plugin_album_securities = freo_security_plugin_album('user', array($_GET['id']));
	$plugin_album_security   = $plugin_album_securities[$_GET['id']];

	if ($plugin_album_security) {
		$plugin_album['title']     = str_replace('[$title]', $plugin_album['title'], $freo->config['plugin']['album']['restriction_title']);
		$plugin_album['file']      = null;
		$plugin_album_files        = null;
		$plugin_album['text']      = str_replace('[$text]', $plugin_album['text'], $freo->config['plugin']['album']['restriction_text']);
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_album_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                    => freo_token('create'),
		'plugin_album'             => $plugin_album,
		'plugin_album_tags'        => $plugin_album_tags,
		'plugin_album_files'       => $plugin_album_files,
		'plugin_album_categories'  => $plugin_album_categories,
		'plugin_album_filter'      => $plugin_album_filter,
		'plugin_album_security'    => $plugin_album_security,
	));

	if (isset($_GET['type']) and $_GET['type'] == 'embed') {
		//データ出力
		freo_output('plugins/album/embed.html');
	}

	return;
}



/* アルバム一覧 */
function freo_page_album_default()
{
	global $freo;

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND title LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%') . ' OR text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['category_id'])) {
		$condition .= ' AND category_id = ' . $freo->pdo->quote($_GET['category_id']);
	}
	if (isset($_GET['tag'])) {
		$condition .= ' AND tag = ' . $freo->pdo->quote($_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote($_GET['tag'] . ',%') . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag']) . ' OR tag LIKE ' . $freo->pdo->quote('%,' . $_GET['tag'] . ',%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}


	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//検索条件
	$ascending = false;

		if ($freo->config['plugin']['album']['album_order'] == 'ascend') {
			$ascending = true;
		}

	if ($ascending == true) {
		$order = ' id';
	} else {
		$order = ' id DESC';
	}

	//アルバム取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums ' . $condition . ' ORDER BY' . $order . ' LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['album']['default_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['album']['default_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_albums = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_albums[$data['id']] = $data;
	}

	//アルバムID取得
	$plugin_album_keys = array_keys($plugin_albums);

	//アルバムタグ取得
	$plugin_album_tags = array();
	foreach ($plugin_album_keys as $plugin_album) {
		if (!$plugin_albums[$plugin_album]['tag']) {
			continue;
		}

		$plugin_album_tags[$plugin_album] = explode(',', $plugin_albums[$plugin_album]['tag']);
	}

	//表示順
	$sort = 0;
	if ($freo->config['plugin']['album']['album_file_sort'] == 'descend') {
		$sort = 1;
	}

	//ファイル取得
	$plugin_album_files = array();
	foreach ($plugin_album_keys as $plugin_album) {
		$file_dir = FREO_FILE_DIR . 'plugins/album_files/' . $plugin_album . '/';
		$thumbnail_dir = FREO_FILE_DIR . 'plugins/album_thumbnails/' . $plugin_album . '/';

		if (file_exists($file_dir)) {
			if ($dir = scandir($file_dir, $sort)) {
				foreach ($dir as $data) {
					if (is_file($file_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
						list($width, $height, $size) = freo_file($file_dir . $data);
						if (is_file($thumbnail_dir . $data) and preg_match("/(\w+)\.\w+$/", $data, $matches)) {
							list($thumbnailwidth, $thumbnailheight, $thumbnailsize) = freo_file($thumbnail_dir . $data);
							$thumbnail = array(
								'name'    => $data,
								'width'   => $thumbnailwidth,
								'height'  => $thumbnailheight,
								'size'    => $thumbnailsize
							);
						} else {
							$thumbnail = array();
						}
						$plugin_album_files[$plugin_album][$matches[1]] = array(
							'name'      => $data,
							'width'     => $width,
							'height'    => $height,
							'size'      => $size,
							'thumbnail' => $thumbnail
						);
					}
				}
			} else {
				freo_error('ファイル保存ディレクトリを開けません。');
			}
		}
	}
	

	//アルバム関連データ取得
	$plugin_album_associates = freo_associate_plugin_album('get', $plugin_album_keys);

	//アルバムフィルター取得
	$plugin_album_filters = freo_filter_plugin_album('user', $plugin_album_keys);

	foreach ($plugin_album_filters as $id => $filter) {
		if (!$filter) {
			continue;
		}

		$plugin_albums[$id]['title']     = str_replace('[$title]', $plugin_albums[$id]['title'], $freo->config['plugin']['album']['filter_title']);
		$plugin_albums[$id]['file']      = null;
		$plugin_album_files[$id]         = null;
		$plugin_albums[$id]['text']      = str_replace('[$text]', $plugin_albums[$id]['text'], $freo->config['plugin']['album']['filter_text']);
	}

	//アルバム保護データ取得
	$plugin_album_securities = freo_security_plugin_album('user', $plugin_album_keys);

	foreach ($plugin_album_securities as $id => $security) {
		if (!$security) {
			continue;
		}

		$plugin_albums[$id]['title']     = str_replace('[$title]', $plugin_albums[$id]['title'], $freo->config['plugin']['album']['restriction_title']);
		$plugin_albums[$id]['file']      = null;
		$plugin_album_files[$id]         = null;
		$plugin_albums[$id]['text']      = str_replace('[$text]', $plugin_albums[$id]['text'], $freo->config['plugin']['album']['restriction_text']);

	}

	//アルバム数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                  = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_album_count = $data[0];
	$plugin_album_page  = ceil($plugin_album_count / $freo->config['plugin']['album']['default_limit']);

	//イラストアルバム取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums WHERE id IS NOT NULL AND category_id = \'illust\' ORDER BY id DESC LIMIT 10');
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_album_illusts = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_illusts[$data['id']] = $data;
	}

	//漫画アルバム取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_albums WHERE id IS NOT NULL AND category_id = \'comic\' ORDER BY id DESC LIMIT 10');
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_album_comics = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_comics[$data['id']] = $data;
	}

	//カテゴリー取得
	$stmt = $freo->pdo->query('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_album_categories ORDER BY sort, id');
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$plugin_album_categories = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_album_categories[$data['id']] = $data;
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'                    => freo_token('create'),
		'plugin_albums'            => $plugin_albums,
		'plugin_album_tags'        => $plugin_album_tags,
		'plugin_album_files'       => $plugin_album_files,
		'plugin_album_count'       => $plugin_album_count,
		'plugin_album_page'        => $plugin_album_page,
		'plugin_album_categories'  => $plugin_album_categories,
		'plugin_album_filters'     => $plugin_album_filters,
		'plugin_album_securities'  => $plugin_album_securities,
		'plugin_album_illusts'     => $plugin_album_illusts,
		'plugin_album_comics'      => $plugin_album_comics,
	));

	return;
}

?>
